﻿if KarmaStub:isLib("LibKarma", 1.1, "$Revision: 13 $") then return end


--[[ 
	Simple ligthweight AddOn framework
]]

LibKarma = LibKarma or {

}


function LibKarma:GetInstance( name, inst )
	local inst = inst or { };
	
	inst._instance = {
		name = name,
		instance = inst,
		libUtility = LibKarmaUtil,
		libLibrary = LibKarma,
		libLocale = LibKarmaLocale,
		libAce = LibKarmaAce,
	}
	
	inst._meta = {
		title = GetAddOnMetadata( name, "Title" ),
		version = GetAddOnMetadata( name, "Version" ), 
		author = GetAddOnMetadata( name, "Author" ),
		desc = GetAddOnMetadata( name, "Description" ),
		icon = GetAddOnMetadata( name, "X-Icon" ),
		lcapture = GetAddOnMetadata( name, "X-LCapture" ) == "1" and 1 or nil,
		locale = GetLocale() == "enGB" and "enUS" or GetLocale(),
	}
	
	inst._update = {
		frame = CreateFrame( "Frame", nil, UIParent ),
		onupdate = function(f, elapsed) f._handler:_OnUpdate(f, elapsed) end,
		time = 0,
		next = 0,
		interval = 0.4,
		delay = 3,
		events = { },		
		gtime = 0,
		timediff = inst._instance.libUtility:GetTimeDiff(),
	}
	inst._update.frame._handler = inst;
	inst._update.frame:SetScript("OnUpdate", inst._update.onupdate );	

	inst._store = {
		db = nil,
		name = nil,
		defaults = { Options = { DebugMode = 1 } },
	}
	inst._options = inst._store.defaults.Options;
	
	local _inst = inst._instance;
	setmetatable( _inst.libLibrary, { __index = _inst.libUtility } );
	return setmetatable( inst, { __index = _inst.libLibrary } );
end


function LibKarma:OnLoad( frame, ... )
	frame:RegisterEvent("VARIABLES_LOADED")
	frame:RegisterEvent("PLAYER_LOGIN")

	if self.OnLoadCustom then self:OnLoadCustom(frame,...) end
end


function LibKarma:OnEvent( frame, event, ... )
	if event == "VARIABLES_LOADED" then
		self:_HookDB();
		if LibKarmaAce and LibKarmaAce.OnVariablesLoaded then LibKarmaAce:OnVariablesLoaded(frame,self); end
		if self.RunVersionFix then self:RunVersionFix(frame); end;
		if self.OnVariablesLoaded then self:OnVariablesLoaded(frame); return; end
	elseif event == "PLAYER_LOGIN" then
		frame:UnregisterEvent("PLAYER_LOGIN");
		if LibKarmaAce and LibKarmaAce.OnPlayerLogin then LibKarmaAce:OnPlayerLogin(frame,self); end
		if self.OnPlayerLogin then self:OnPlayerLogin(frame); return; end
	elseif event == "UPDATE_MOUSEOVER_UNIT"	then
		if self.OnUpdateMouse then self:OnUpdateMouse(frame,...); return; end
	end	
	if self._update.events then										
		tinsert(self._update.events, {frame, event, ...} );			-- self:Debug("Queueing", self._update.time, frame and frame:GetName(), event, ...);
		return;
	end
	if self.OnEventCustom then self:OnEventCustom(frame,event,...) end
end


function LibKarma:_OnUpdate( frame, elapsed, ... )
	local u = self._update 
	u.time = u.time + elapsed;
	if u.time < u.next then	return else	u.next = u.time + u.interval end	
	if u.events and u.time > u.delay then
		for k, v in pairs(u.events) do								-- self:Debug("Running", u.time, unpack(v));
			self:OnEventCustom(unpack(v));							
		end
		u.events = nil;
	end
	u.gtime = self:GetGameTime();
	if u.glast ~= u.gtime then
		u.glast = u.gtime;
		u.timediff = self:GetTimeDiff(u.gtime)						-- self:Debug("Updating", u.timediff);
	end
	if self.UpdateFrame_OnUpdate then self:UpdateFrame_OnUpdate(frame, elapsed, ...) end
end

function LibKarma:SafeCall( f, ... )
	local e, r = pcall( f, ... );
	if not e then self:Debug("SafeCall exception", r); return nil end
	return r;
end

function LibKarma:_HookTablesDB( db, defaults, recurse )
	for i,v in pairs(defaults) do
		if type(v) == "table"  then
			if not rawget(db, i) then rawset(db, i, { }) end 
			if recurse then self:_HookTablesDB( db[i], defaults[i], recurse ) end
			setmetatable(rawget(db, i), {__index = rawget(defaults, i)});
		end
	end
	return db;
end 

function LibKarma:_GetDB( db, name, defaults, recurse )
	db = name and _G[name] or db or { };
	if name then _G[name] = db end
	self:_HookTablesDB( db, defaults, recurse )
	return setmetatable(db, {__index = defaults});
end

function LibKarma:_HookDB( )
	local s = self._store;	
	s.db = self:_GetDB( s.db, s.name, s.defaults );
	self._options = s.db.Options
	self._db = s.db
end 

function LibKarma:RegisterDB( name, defaults )
	local s = self._store;
	s.name = name;
	s.defaults = defaults or s.defaults or { };		
	self:_HookDB( );
end 

function LibKarma:ImportRenameDB( t, names )
	for i,v in pairs(names) do
		if rawget(t,v) then 
			if type(i) ~= "number" and not rawget(t,i) then 
				t[i] = t[v];
			end
			t[v] = nil;
		end
	end
end

function LibKarma:ImportToDB( sub, src, prefix, isMerge, isDelete, names  )
	if not names then return end
	local dst = self._store.db;
	src = src or dst; 
	
	if sub then 
		if not dst[sub] then dst[sub] = { }  end
		dst = dst[sub];
	end

	for i,v in pairs(names) do
		if type(i) == "number" then i = v end
		local s = (prefix or "")..v
		if src[s] then
			if isMerge then
				self:MergeDeep( dst[i], src[s], false );
			elseif not rawget(dst,i) then
				dst[i] = self:CopyDeep( src[s] );
			end
			if isDelete then src[s]=nil end
		end
	end
end
